package cn.ecpark.service.usergw.biz.filters.factory;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSON;

import org.apache.dubbo.config.ApplicationConfig;
import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.cloud.gateway.support.HttpStatusHolder;
import org.springframework.cloud.gateway.support.ServerWebExchangeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;

import cn.ecpark.service.usergw.biz.filters.bean.GenericServicePool;
import reactor.core.publisher.Mono;;

@Component
public class DubboGatewayFilterFactory extends AbstractGatewayFilterFactory<DubboGatewayFilterFactory.Config> {

    @Autowired
    private ApplicationContext appContext;

    /**
     * DUBBO_URI key.
     */
    public static final String DUBBO_URI = "dubbo_uri";

    public DubboGatewayFilterFactory() {
        super(Config.class);
    }

    @Override
    public List<String> shortcutFieldOrder() {
        return Arrays.asList(DUBBO_URI);
    }

    @Override
    public GatewayFilter apply(Config config) {
        String uri = config.dubboUri;

        return (exchange, chain) -> {

            GenericServicePool gsPool = appContext.getBean(GenericServicePool.class);
            GenericService gs = gsPool.get(uri);
            // special
            Object result = gs.$invoke("Say", new String[] { "java.lang.String" }, new Object[] { "222" });
            // gsPool.add(rkey, gs);
            if (result != null) {
                ServerHttpResponse response = exchange.getResponse();
                return response.writeWith(Mono
                        .just(response.bufferFactory().wrap(ByteBuffer.wrap(JSON.toJSONString(result).getBytes()))));
            }

            return chain.filter(exchange);
        };
    }

    public static class Config {

        private String dubboUri;

        public String getDubboUri() {
            return dubboUri;
        }

        public void setDubboUri(String dubboUri) {
            this.dubboUri = dubboUri;
        }

    }

}